/*
	Fractal - A program to calculate Newton fractals.
	Copyright (C) 2004 Johannes Bauer
	
	This file is part of Fractal.

    Fractal is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Fractal is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Fractal; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Johannes Bauer
	Mussinanstr. 140
	92318 Neumarkt/Opf.
	JohannesBauer@gmx.de
*/

#include <vector>
#include <errno.h>
#include <string.h>
#include <stdio.h>

#include "Fractal.hpp"
#include "CPoly.hpp"
#include "CTexExport.hpp"

CTexExport::CTexExport() {
	this->Polynomial=NULL;
}

void CTexExport::Write(const char *Filename) {
	FILE *f;
	if (this->Polynomial==NULL) {
		fprintf(stderr,"No polynomial was set, so '%s' could not be written.\n",Filename);
		return;
	}
	f=fopen(Filename,"w");
	if (f==NULL) {
		fprintf(stderr,"Could not open file '%s' for writing: %s\n",Filename,strerror(errno));
		return;
	}
//	fprintf(f,"Tex docuemnt:\n");
//	fprintf(f,"Number of coefficients: %d\n",this->Polynomial->CoefficientCount());

	fprintf(f,"\\documentclass[12pt]{scrartcl}\n");
	fprintf(f,"\\usepackage[T1]{fontenc}\n");
	fprintf(f,"\\usepackage[latin1]{inputenc}\n");
	fprintf(f,"\\usepackage{graphicx}\n");
	fprintf(f,"\\usepackage{pst-all,pstricks,pst-plot}\n");
	fprintf(f,"\\usepackage{multido}\n");
	fprintf(f,"\\usepackage{fancybox,amssymb,color}\n");
	fprintf(f,"\\usepackage{geometry}\n");
	fprintf(f,"\n");
	fprintf(f,"\\geometry{paper=a4paper,left=1cm,right=1cm,top=1.5cm,bottom=1.5cm,headheight=0cm,marginparwidth=3cm}\n");
	fprintf(f,"\n");
	fprintf(f,"\\begin{document}\n");
	fprintf(f,"\n");
	fprintf(f,"\\setlength{\\parindent}{0cm}\n");
	fprintf(f,"\n");
	fprintf(f,"\\sffamily\n");
	fprintf(f,"\\Large\n");
	fprintf(f,"Fractal Calculation Results\n");
	fprintf(f,"\\normalsize\n");
	fprintf(f,"\n");
	fprintf(f,"\\ \\\\\n");
	fprintf(f,"\n");
	fprintf(f,"The selected polynomial was:\\\\\n");

	fprintf(f,"$");
	for (int i=this->Polynomial->CoefficientCount()-1;i>=0;i--) {
		if (i!=this->Polynomial->CoefficientCount()-1) {
			// Not the first coefficient
			if (this->Polynomial->GetCoefficient(i).real()>0) {
				fprintf(f,"+ ");
			}
		}
		if ((i!=1) && (i!=0)) {
			fprintf(f,"%.46f \\cdot x^{%d}\\\\\n",this->Polynomial->GetCoefficient(i).real(),i);
		} else {
			if (i==1) {
				fprintf(f,"%.46f \\cdot x\\\\\n",this->Polynomial->GetCoefficient(i).real());
			} else {
				fprintf(f,"%.46f\\\\\n",this->Polynomial->GetCoefficient(i).real());
			}
		}
	}
	fprintf(f,"$\n");
	
	fprintf(f,"\n");
	fprintf(f,"The used drawing range was:\\\\\n");
	fprintf(f,"$R = (%.46f,\\\\\n%.46f)$\\\\\n",this->MinR,this->MaxR);
	fprintf(f,"$C = (%.46f,\\\\\n%.46f)$\\\\\n",this->MinI,this->MaxI);
	fprintf(f,"\n");
	fprintf(f,"\\begin{figure}[h]\n");
	fprintf(f,"\t\\centering\n");
	fprintf(f,"\t\\includegraphics[width=17cm]{Out.eps}\n");
	fprintf(f,"\t\\caption{Fractal image}\n");
	fprintf(f,"\t\\label{img:fractalimage}\n");
	fprintf(f,"\\end{figure}\n");
	fprintf(f,"\n");
	
	fprintf(f,"\\psset{unit=0.9cm}\n");

	fprintf(f,"\\begin{pspicture}(-10,-10)(10,10)\n");
	fprintf(f,"\t\\centering\n");
	fprintf(f,"\t\\psclip{\\psframe[linecolor=white](-10,-10)(10,10)}\n");
	fprintf(f,"\t\t\\psaxes{->}(0,0)(-10,-10)(10,10)\n");

	fprintf(f,"\t\t\\psplot[plotstyle=curve]{-10}{10}{");
	
	// Now write the term in inverse polish notation (UPN)
	// This is nasty...
	for (int i=0;i<this->Polynomial->CoefficientCount();i++) {
		fprintf(f,"%f ",this->Polynomial->GetCoefficient(i).real());
		if (i>0) {
			for (int j=1;j<=i;j++) {
				fprintf(f,"x mul ");
			}
			fprintf(f,"add ");
		}
	}
			
	fprintf(f,"}\n");

	fprintf(f,"\t\t\\rput(9.5,0.2){$\\mathbf{x}$}\n");
	fprintf(f,"\t\t\\rput(0.3,0.5){$\\mathbf{y}$}\n");
	
	fprintf(f,"\t\\endpsclip\n");
	fprintf(f,"\\end{pspicture}\n");
	
	fprintf(f,"\n");
	fprintf(f,"\\end{document}\n");
	
	fclose(f);
}

void CTexExport::Set_Coordinates(double MinR, double MinI, double MaxR, double MaxI) {
	this->MinR=MinR;
	this->MinI=MinI;
	this->MaxR=MaxR;
	this->MaxI=MaxI;
}

void CTexExport::Set_Polynomial(CPoly *Polynomial) {
	this->Polynomial=Polynomial;
}

CTexExport::~CTexExport() {
}

